/************************************
CSE142, Spring 1999
Homework 4
Traffic animation/Airport Landing Sim

Name: Andrew McComas

Date: 5-21-99
Class: CSE 142 AF
*************************************/


#include "gp142.h"
#include <stdio.h>
#include <math.h>

/* Logical constants */

#define TRUE    		1
#define FALSE   		0

/* 
 * For many of the GP142 functions, a line width of
 * zero means to fill the shape being drawn.  The
 * constant FILL is used in function calls when a solid
 * shape is desired.
 */
#define FILL 0

/* Menu buttons: */
/* Top-left coordinates of button palette */
#define MENU_TOP		(GP142_YMAX - 5)
#define MENU_LEFT		(-GP142_XMAX + 5)

/* Height and width of buttons in pixels */
#define BUTTON_HEIGHT  	25
#define BUTTON_WIDTH 	75

/* Number of buttons */
#define NUM_BUTTONS 	4


/* Possible locations of mouse clicks: */
/*   Mouse click in some button */
#define BUTTON_CLICK 1

/*   Mouse click elsewhere */
#define OTHER_CLICK 0


/* Coordinates of objects in the drawing */
/* y-coordinate of the top of the ground in the landscape */
#define HORIZON (- 2 * (GP142_YMAX / 10))


/* Maximum number of vehicles that can appear in the animation */
#define MAX_TRAFFIC	9

/* Height and width of a single car */
#define CAR_HEIGHT 24
#define CAR_WIDTH  40

/* Window Constants for Buildings */
#define WINDOW_SPACING_X 3
#define WINDOW_SPACING_Y 3
#define WINDOW_SIZE     1


/*****************************************************************************

     FUNCTION PROTOTYPES

*****************************************************************************/

/* Draw the background, including the sun at coordinates sun_x and sun_y */
void draw_landscape(int sun_x, int sun_y);

/* Draw the buttons */
void draw_buttons();

/* Return kind of mouse click (in a button, etc.) */
int  classify_mouse_click(int mouse_x, int mouse_y);

/* Process button click at location x, y.  Update car speed,  */
/* direction, color, and whether traffic is moving as needed. */
void handle_button(int x, int y, int *traffic_moving, int *car_speed,
				   int *car_direction, int *car_color);

/* Update sun coordinates sun_x, sun_y for one tick */
void move_sun(int *sun_x, int *sun_y);

/* Animate the air traffic */
void move_air_traffic(int air_traffic_x[MAX_TRAFFIC], 
					  int *y, int *direction, int speed);

/* Update all vehicle x, y coordinates, given current x, y */
/* location, direction (+1 or -1), and speed in pixels per tick. */
void move_traffic(int *x, int *y, int *speed);

/* Draw all vehicles given their current x, y coordinates and color */
void draw_traffic(int x, int y, int color);

/* Calls the small airplane function */
void draw_air_traffic(int air_traffic_x[MAX_TRAFFIC], int y, int color);

/* Draw a single car of the given color with horizontal */
/* center x and bottom y*/
void draw_airplane(int x, int y, int color);

/* Draws the distant airplanes */
void draw_small_airplane(int x, int y, int color);

/* Draws a building with windows */
void draw_building(int distance, int width, int height, int color);

/* Draws the Runway */
void draw_runway(int color);

/* Display error message if internal error is detected. */
void something_is_wrong (void);


/*****************************************************************************

     MAIN

*****************************************************************************/

int main(void)

{
	/* Event handling and buttons: */
	int  quit;				/* = "user has selected quit"				*/
	int  event;				/* the most recent GP142 event				*/
	char key_pressed;		/* last keyboard character from key event	*/
	int  mouse_x, mouse_y;	/* x and y coordinates of latest mouse event*/
	int  click_kind;		/* kind of mouse click (button, etc.)		*/

	/* Information about the sun: */
	int sun_x, sun_y;			/* screen x and y coordinates */

	/* Information about vehicles in the animation:	*/
	/* [Only one car now - will need to be changed to add more]	*/
	int air_traffic_x[MAX_TRAFFIC]; /* Array for non controlled planes */
	int air_traffic_y;
	int air_traffic_direction;
	int air_traffic_speed;
	int i;

	
	int plane_x, plane_y;		/* current position of the car	*/
	int car_color;			/* color of the car	*/
	int plane_speed;			/* speed in pixels per tick when moving	*/	
	int traffic_moving;		/* = "traffic is currently moving" */

	/* Initialize graphics package, clear the window,	*/
	/* turn logging off, and start animation.			*/
	GP142_open();
	GP142_logging(LOG_OFF);
	GP142_clear();
	GP142_animate(ANI_RUN);

	/* set initial locations of air traffic */
	for (i = 0; i < MAX_TRAFFIC; i++)
	{
		air_traffic_x[i] = -i * 75;
	}

	air_traffic_y = GP142_YMAX - 30;
	air_traffic_direction = -1;
	air_traffic_speed = 5;

	/* set initial location of the sun */
	sun_x = GP142_XMAX - 50;
	sun_y = GP142_YMAX - 35;

	/* set initial conditions for the plane */
	plane_x = GP142_XMAX - 30;
	plane_y = HORIZON + 90;
	plane_speed = 12; 
	car_color = RED;
	traffic_moving = TRUE;
	
	/* Draw initial landscape, buttons, and traffic */
	draw_landscape(sun_x,sun_y);
	draw_buttons( );
	draw_traffic(plane_x, plane_y, car_color);


	/*
	 * Main event loop:
	 * ---- ----- -----
	 * Wait for the next user action, decode it, and call an
	 * appropriate function to handle it.  Repeat until the
	 * user selects quit.
	 */

	quit = FALSE;
	while (!quit) {
		
		/* get next event */
		event = GP142_await_event(&mouse_x, &mouse_y, &key_pressed);
		
		/* perform appropriate action depending on kind of event */
		switch (event) {
		case GP142_QUIT:
			/* Quit selected */
			quit = TRUE;
			break;
			
		case GP142_KBD:
			/* Key pressed.  Not used now.  If the program should		*/
			/* react to keyboard events, insert something appropriate	*/
			/* here.  Note that scanf CANNOT be used with GP142.		*/
			break;
			
		case GP142_MOUSE:
			/* Mouse click.  Determine if in a menu button	*/
			/* or elsewhere and process accordingly.		*/
			click_kind = classify_mouse_click(mouse_x, mouse_y);
			if (click_kind == BUTTON_CLICK) {
				handle_button(mouse_x, mouse_y, &plane_x, &plane_y,
								&plane_speed, &car_color);
			} else {
				/* insert code here (and additional else-if statements) */
				/* to react to mouse clicks elsewhere on the screen */
			}
			break;
			
		case GP142_PERIODIC:
			/* Timer interval event.  Update everything	*/
			/* that needs to change as time advances.	*/
			move_sun(&sun_x, &sun_y);
			
			/* Animate the airplanes in the sky */
			move_air_traffic(air_traffic_x, &air_traffic_y, 
				&air_traffic_direction, air_traffic_speed);

			move_traffic(&plane_x, &plane_y, &plane_speed);
				
			break;
			
		default:
			/* unknown event */
			break;
			
		}  /*end switch */
		
		/* Redraw everything */
		draw_landscape(sun_x, sun_y);
		draw_buttons();
		
		/* Draw Landing Plane */
		draw_traffic(plane_x, plane_y, car_color);
		
		/* Conditions and status */
		if (plane_speed == 0 && plane_x > -GP142_XMAX + 210) 
				GP142_printfXY(RED, plane_x + 30, plane_y, 19, 
				"CRASH!!!!!");
		if (plane_speed == 0 && plane_x < -GP142_XMAX + 210 && 
			plane_x > -GP142_XMAX) 
				GP142_printfXY(BLUE, plane_x + 30, plane_y, 19, 
				"NICE LANDING");
		if (plane_speed != 0 && plane_x < -GP142_XMAX) 
				GP142_printfXY(RED, -GP142_XMAX + 10, 
				plane_y, 19, "OVERSHOT RUNWAY");
		
		/* Draw airplanes in the sky */
		draw_air_traffic(air_traffic_x, air_traffic_y, car_color);

	}  /* end event loop */
	
	
	/* Termination: shut down graphics window and exit */
	GP142_close();
	return 0;
}  /*end main */


/*****************************************************************************

     BACKGROUND LANDSCAPE

*****************************************************************************/


/* Draw the background, including the sun at coordinates sun_x and sun_y */
void draw_landscape(int sun_x, int sun_y)
{
	int text_offset = -GP142_XMAX + 15;	/* horizontal offset of text */
	int text_size   = 12;				/* size of text font */
	

	/* Draw the sky */
	GP142_rectangleXY(ICE_BLUE, -GP142_XMAX, -GP142_YMAX,
		GP142_XMAX, GP142_YMAX, FILL);
	
	/* Draw the sun */
	GP142_circleXY(YELLOW, sun_x, sun_y , 20);
	
	/* Draw the ground, highway, and text legend */
	GP142_rectangleXY(BROWN, -GP142_XMAX, -GP142_YMAX,
		GP142_XMAX, HORIZON, FILL);
	
	/* Draw the Runway */
	draw_runway(BLUE);

	GP142_printfXY (BLACK, text_offset, -GP142_YMAX+30, 15,
		"Safely land the airplane.");
		
	/*Draw some mountains */
	GP142_triangleXY(MED_GRAY,
		-GP142_XMAX, HORIZON,
		GP142_XMAX, HORIZON,
		0, (4 * GP142_YMAX)/5 ,
		FILL);
	
	GP142_triangleXY(OLIVE,
		-( 5 * GP142_XMAX/4), HORIZON,
		(GP142_XMAX)/3  , HORIZON,
		-(5 * GP142_XMAX)/4, (2 * GP142_YMAX)/3 ,
		FILL);
	
	GP142_triangleXY(FOREST_GREEN,
		-GP142_XMAX / 3 , HORIZON,
		GP142_XMAX, HORIZON,
		GP142_XMAX, (2 * GP142_YMAX)/3 ,
		FILL); 

	/* Draw Buildings */
	/* Distance from left, Width divisible by four, Height, Color */
	draw_building(30, 32, 80, BLACK);
	draw_building(70, 32, 90, BLACK);
	draw_building(110, 32, 85, MED_GRAY);
	draw_building(150, 32, 95, BLACK);
	draw_building(190, 32, 130, BLACK);
	draw_building(230, 32, 170, BLACK);

	/* Draw Antenna on Building */
	GP142_lineXY(RED, -GP142_XMAX + 240, HORIZON + 170, 
		-GP142_XMAX + 240, HORIZON + 190, 2);
	GP142_lineXY(RED, -GP142_XMAX + 248, HORIZON + 170, 
		-GP142_XMAX + 248, HORIZON + 180, 2);
	GP142_lineXY(RED, -GP142_XMAX + 256, HORIZON + 170, 
		-GP142_XMAX + 256, HORIZON + 190, 2);

	draw_building(270, 32, 120, BLACK);
	draw_building(310, 32, 100, BLACK);
	draw_building(350, 32, 80, MED_GRAY);
	draw_building(390, 32, 90, BLACK);
	draw_building(430, 32, 70, BLACK);
	draw_building(470, 32, 40, MED_GRAY);
	draw_building(510, 64, 35, BLACK);

} 


/* Draw a Building with windows */
void draw_building(int distance, int width, int height, int color) {

	int spacer_x = WINDOW_SPACING_X;
	int spacer_y = WINDOW_SPACING_Y;

	/* Actual Building */
	GP142_rectangleXY(color, -GP142_XMAX + distance, HORIZON, 
		-GP142_XMAX + distance + width, HORIZON + height, FILL); 

	/* Draw Windows row by row */
	while(spacer_y < height - WINDOW_SIZE && height > 10 + spacer_y) {
		
		while(spacer_x < width - WINDOW_SIZE) { 
			
			
			GP142_rectangleXY(YELLOW, -GP142_XMAX + distance + spacer_x, 
				HORIZON + height - spacer_y, 
				-GP142_XMAX + distance + spacer_x + WINDOW_SIZE, 
				HORIZON + height - WINDOW_SIZE - spacer_y, FILL);
			spacer_x += (1 + WINDOW_SPACING_X) + WINDOW_SIZE;
			
		}
		
		spacer_x = WINDOW_SPACING_X;
		spacer_y += (1 + WINDOW_SPACING_Y) + WINDOW_SIZE;
	}

}


/* Draws the Runway and Runway Lights */
void draw_runway(int color) {

	int x, y;


	GP142_rectangleXY(LT_GRAY, -GP142_XMAX, HORIZON - 40, 
		-GP142_XMAX + 210, HORIZON - 90, FILL);
	GP142_rectangleXY(BLACK, -GP142_XMAX, HORIZON - 40, 
		-GP142_XMAX + 210, HORIZON - 90, 1);


	GP142_lineXY(YELLOW, -GP142_XMAX, HORIZON - 65, 
		-GP142_XMAX + 20, HORIZON - 65, 1);
	GP142_lineXY(YELLOW, -GP142_XMAX + 30, HORIZON - 65, 
		-GP142_XMAX + 50, HORIZON - 65, 1);
	GP142_lineXY(YELLOW, -GP142_XMAX + 60, HORIZON - 65, 
		-GP142_XMAX + 80, HORIZON - 65, 1);
	GP142_lineXY(YELLOW, -GP142_XMAX + 90, HORIZON - 65, 
		-GP142_XMAX + 110, HORIZON - 65, 1);
	GP142_lineXY(YELLOW, -GP142_XMAX + 120, HORIZON - 65,
		-GP142_XMAX + 140, HORIZON - 65, 1);
	GP142_lineXY(YELLOW, -GP142_XMAX + 150, HORIZON - 65, 
		-GP142_XMAX + 170, HORIZON - 65, 1);

	/* Draw End of Runway Lines */
	GP142_lineXY(WHITE, -GP142_XMAX + 205, HORIZON - 45, 
		-GP142_XMAX + 205, HORIZON - 85, 2);
	GP142_lineXY(WHITE, -GP142_XMAX + 200, HORIZON - 45, 
		-GP142_XMAX + 200, HORIZON - 85, 2);
	GP142_lineXY(WHITE, -GP142_XMAX + 195, HORIZON - 45, 
		-GP142_XMAX + 195, HORIZON - 85, 2);

	/* Draw End of Runway Warning Lights */
	x = -GP142_XMAX + 215;
	y = HORIZON - 45;
	
	while (y > HORIZON - 90) {
	
	GP142_circleXY(RED, x, y, 1);
	
	y = y - 8;

	}


	/* Draw Runway Lights */
	x = -GP142_XMAX + 2;
	y = HORIZON - 35;
	
	while (x < -GP142_XMAX + 210) {
	
		GP142_circleXY(color, x, y, 1);
		x = x + 8;

	}

	
	x = -GP142_XMAX + 2;
	y = HORIZON - 95;
	
	while (x < -GP142_XMAX + 210) {
	
		GP142_circleXY(color, x, y, 1);
		x = x + 8;
	}

}

/*****************************************************************************

     BUTTONS AND MOUSE CLICKS

*****************************************************************************/


/* Draws the buttons */
void draw_buttons( )
{
	int i;
	int text_offset = MENU_LEFT+10;	/* offset of button text in button */
	int font_size = 12;				/* size of button text */
	
	/* draw rectangles for each button */
	for(i = 0; i < NUM_BUTTONS; i++) {
		GP142_rectangleXY(BLACK,
			MENU_LEFT,
			MENU_TOP - i * BUTTON_HEIGHT,
			MENU_LEFT + BUTTON_WIDTH,
			MENU_TOP - (i + 1)*BUTTON_HEIGHT, 1);
	}
	
	/* display button text */
	i = MENU_TOP - (BUTTON_HEIGHT/2) - (font_size/2);
	GP142_printfXY(FOREST_GREEN, text_offset, i, font_size, "LAND");	
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(RED, text_offset, i, font_size, "PULL UP");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "BRAKES");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "RESTART");
	

}

/* Classify mouse click at (mouse_x,mouse_y) and return kind */
int  classify_mouse_click(int mouse_x, int mouse_y)
{
	if ((mouse_x > MENU_LEFT) &&
		(mouse_x < MENU_LEFT + BUTTON_WIDTH) &&
		(mouse_y < MENU_TOP) &&
		(mouse_y > (MENU_TOP - NUM_BUTTONS*BUTTON_HEIGHT)))
		return BUTTON_CLICK;
	else
		return OTHER_CLICK;
}



/* Process button click at location x, y.  Update car speed,	*/
/* direction, color, and whether traffic is moving as needed.	*/
void handle_button(int x, int y, int *plane_x, int *plane_y,
				   int *plane_speed, int *car_color)
{
	int buttonNum;		/* number of selected button */

	/* Complain if x coordinate couldn't be inside a button */
	if (x < MENU_LEFT || x > MENU_LEFT + BUTTON_WIDTH) {
		something_is_wrong();
		return;
	}
	
	/* Convert y coordinate to button number from top. */
	buttonNum = (MENU_TOP - y)/BUTTON_HEIGHT;	 
	
	switch(buttonNum) {
	case 0:
		/* LAND */
		if (*plane_speed != 0 || *plane_y > HORIZON - 65) {
		*plane_x = *plane_x - 5;
		*plane_y = *plane_y - 5;
		
		}
		
		break;
	case 1:
		/* TAKEOFF */ 
		if (*plane_speed != 0 || *plane_y > HORIZON - 65) {
		*plane_x = *plane_x - 5;
		*plane_y = *plane_y + 5;
		}
		else
		GP142_printfXY(RED, *plane_x + 20, *plane_y - 20, 14, "CRASH");
		break;
		
	case 2:
		/* BRAKES */ 
		if (*plane_y < HORIZON - 65 && *plane_y > HORIZON - 90)
			*plane_speed = *plane_speed - 1; 

		if (*plane_speed < 0) {
			*plane_speed = 0;

		}

		break;
	case 3:
		*plane_x = 	GP142_XMAX - 30;
		*plane_y = HORIZON + 90;
		*plane_speed = 10;

		break;

	default:
		/*Bad news if this is ever reached! */
		something_is_wrong();
		break;
	}
}



/*****************************************************************************

     TRAFFIC AND OTHER ANIMATED OBJECTS

*****************************************************************************/


/* Update sun coordinates sun_x, sun_y by one tick */
void move_sun(int *sun_x, int *sun_y) {
	/* Move the sun to the left.  If it reaches the edge, move it */
	/* to the other side and down a bit. */
	(*sun_x) --;
	if (*sun_x < -GP142_XMAX -30) {
		*sun_x = - *sun_x;
		(*sun_y) --;
	}
}


/* Function to control movement of air trafic */
void move_air_traffic(int air_traffic_x[MAX_TRAFFIC], int *y, 
					  int *direction, int speed) {


	int trial_x;
	int i;

	for (i = 0; i < MAX_TRAFFIC; i++)
	{

		trial_x = air_traffic_x[i] + (*direction * speed);

		if (trial_x > GP142_XMAX)
			air_traffic_x[i] = -GP142_XMAX;

		else if (trial_x < -(GP142_XMAX))
			air_traffic_x[i] = GP142_XMAX;

		else
			air_traffic_x[i] = trial_x;
	}

}


/* Update all vehicle x, y coordinates, given current x, y */
/* location, direction (+1 or -1), and speed in pixels per tick, */
void move_traffic(int *x, int *y, int *speed) {
	/* Note: If there is more than one vehicle, this should */
	/* be updated to adjust all vehicle coordinates */

	int trial_x;

	if (*x > -GP142_XMAX + 210)

		/* If Plane Reaches ground before runway, stop moving */
		if (*y < HORIZON - 65)
			*speed = 0; 


	trial_x = *x - *speed; 

	if (abs(trial_x) > GP142_XMAX + CAR_WIDTH) 
		*x = - trial_x;
			
	else
		*x = trial_x;

	

}


/* Draw Landing Vehicle given current x, y coordinates and color */

void draw_traffic(int x, int y, int color) {
	/* Note: needs to be changed if there is more than */
	/* one vehicle */

	draw_airplane(x,y, color);
}

/* Draw aircraft in sky */
void draw_air_traffic(int air_traffic_x[MAX_TRAFFIC], 
					  int y, int color) {
	/* Note: needs to be changed if there is more than */
	/* one vehicle */

	int i;

	for (i = 0; i < MAX_TRAFFIC; i++) {

		draw_small_airplane(air_traffic_x[i],y, color);
	}

}


/* Draw a single plane of the given color with horizontal */
/* center x and bottom y*/
void draw_airplane(int x, int y, int color) {


	GP142_ovalXY(WHITE, x - CAR_WIDTH/2, y + CAR_HEIGHT/4,
						x + CAR_WIDTH/2, y + CAR_HEIGHT*1/2, FILL);
	

	GP142_circleXY(BLACK, x - CAR_WIDTH/4, y + CAR_HEIGHT/6, CAR_HEIGHT/7);
	GP142_circleXY(BLACK, x + CAR_WIDTH/4, y + CAR_HEIGHT/6, CAR_HEIGHT/7);
	

	GP142_triangleXY(WHITE, x + CAR_WIDTH/2, y + CAR_HEIGHT/3,
							x + CAR_WIDTH/2 - 7, y + CAR_HEIGHT/3,
							x + CAR_WIDTH/2, y + CAR_HEIGHT/3 + 10, FILL);
}


/* Draw distant airplanes */
void draw_small_airplane(int x, int y, int color) {


	GP142_ovalXY(WHITE, x - CAR_WIDTH/2, y + CAR_HEIGHT/4,
						x + CAR_WIDTH/2, y + CAR_HEIGHT*1/2, FILL);
	


	GP142_triangleXY(WHITE, x + CAR_WIDTH/2, y + CAR_HEIGHT/3,
							x + CAR_WIDTH/2 - 7, y + CAR_HEIGHT/3,
							x + CAR_WIDTH/2, y + CAR_HEIGHT/3 + 10, FILL);
}

/*****************************************************************************

     OTHER

*****************************************************************************/



/* Display error message if internal error is detected. */
void something_is_wrong (void) {
	GP142_printfXY (RED, 0, 0, 16, "BAAAADDD NEWS!");
}